@@ -20,6 +20,7 @@ require 'test/unit/data'
 require 'test/unit/testsuite'
 require 'test/unit/test-suite-creator'
 require 'test/unit/assertion-failed-error'
+require 'test/unit/auto-runner-loader'
 require 'test/unit/util/backtracefilter'
 require 'test/unit/util/output'
 require 'test/unit/util/method-owner-finder'
@@ -36,49 +37,51 @@ module Test
     # You can run two hooks before/after a TestCase run.
     #
     # Example:
-    #   class TestMyClass < Test::Unit::TestCase
-    #     class << self
-    #       def startup
+    #
+    #     class TestMyClass < Test::Unit::TestCase
+    #       class << self
+    #         def startup
+    #           ...
+    #         end
+    #
+    #         def shutdown
+    #           ...
+    #         end
+    #       end
+    #
+    #       def setup
     #         ...
     #       end
     #
-    #       def shutdown
+    #       def cleanup
+    #         ...
+    #       end
+    #
+    #       def teardown
+    #         ...
+    #       end
+    #
+    #       def test_my_method1
+    #         ...
+    #       end
+    #
+    #       def test_my_method2
     #         ...
     #       end
     #     end
     #
-    #     def setup
-    #       ...
-    #     end
-    #
-    #     def cleanup
-    #       ...
-    #     end
-    #
-    #     def teardown
-    #       ...
-    #     end
-    #
-    #     def test_my_method1
-    #       ...
-    #     end
-    #
-    #     def test_my_method2
-    #       ...
-    #     end
-    #   end
-    #
     # Here is a call order:
-    # * startup
-    # * setup
-    # * test_my_method1
-    # * cleanup
-    # * teardown
-    # * setup
-    # * test_my_method2
-    # * cleanup
-    # * teardown
-    # * shutdown
+    #
+    # 1. startup
+    # 1. setup
+    # 1. test_my_method1
+    # 1. cleanup
+    # 1. teardown
+    # 1. setup
+    # 1. test_my_method2
+    # 1. cleanup
+    # 1. teardown
+    # 1. shutdown
     class TestCase
       include Attribute
       include Fixture
@@ -104,37 +107,41 @@ module Test
 
       class << self
         def inherited(sub_class) # :nodoc:
-          require "test/unit"
           DESCENDANTS << sub_class
           super
         end
 
-        @@added_methods = {}
+        @@added_method_names = {}
         def method_added(name) # :nodoc:
           super
-          _added_methods = added_methods
+          added_method_names = (@@added_method_names[self] ||= {})
           stringified_name = name.to_s
-          if _added_methods.include?(stringified_name)
+          if added_method_names.key?(stringified_name)
             attribute(:redefined, {:backtrace => caller}, {}, stringified_name)
           end
-          _attributes = attributes_table[stringified_name] || {}
-          source_location = _attributes[:source_location]
+          source_location = find_attribute(stringified_name, :source_location)
           if source_location
             path, line = source_location
+          elsif respond_to?(:caller_locations, true)
+            location = caller_locations(1, 1)[0]
+            path = location.absolute_path || location.path
+            line = location.lineno
           else
-            path, line, = caller[0].split(/:(\d+)/,2)
+            # TODO: Remove me when Ruby 1.9 support is dropped
+            path, line, = caller[0].split(/:(\d+)/, 2)
             line = line.to_i if line
           end
           method_locations << {
             :method_name => stringified_name,
-            :path => path,
+            :path => File.expand_path(path),
             :line => line,
           }
-          _added_methods << stringified_name
+          added_method_names[stringified_name] = true
+          AutoRunnerLoader.check(self, stringified_name)
         end
 
-        def added_methods # :nodoc:
-          @@added_methods[self] ||= []
+        def added_method_names # :nodoc:
+          (@@added_method_names[self] ||= {}).keys
         end
 
         # Rolls up all of the test* methods in the fixture into
@@ -223,7 +230,11 @@ module Test
         # Returns the current test order. This returns
         # +:alphabetic+ by default.
         def test_order
-          @@test_orders[self] || AVAILABLE_ORDERS.first
+          ancestors.each do |ancestor|
+            order = @@test_orders[ancestor]
+            return order if order
+          end
+          AVAILABLE_ORDERS.first
         end
 
         # Sets the current test order.
@@ -282,6 +293,9 @@ module Test
           else
             targets = test_description_or_targets
             attribute(:test, true, {}, *targets)
+            targets.each do |target|
+              AutoRunnerLoader.check(self, target)
+            end
           end
         end
 
@@ -306,7 +320,7 @@ module Test
         # the same in meaning:
         #
         # Standard:
-        #   class TestParent < Test::UnitTestCase
+        #   class TestParent < Test::Unit::TestCase
         #     class TestChild < self
         #       def test_in_child
         #       end
@@ -314,14 +328,14 @@ module Test
         #   end
         #
         # Syntax sugar:
-        #   class TestParent < Test::UnitTestCase
+        #   class TestParent < Test::Unit::TestCase
         #     sub_test_case("TestChild") do
         #       def test_in_child
         #       end
         #     end
         #   end
         #
-        # The diffrence of them are the following:
+        # The difference of them are the following:
         #
         # * Test case created by {sub_test_case} is an anonymous class.
         #   So you can't refer the test case by name.
@@ -332,21 +346,22 @@ module Test
         #
         # @param name [String] The name of newly created sub test case.
         # @yield
-        #   The block is evaludated under the newly created sub test
+        #   The block is evaluated under the newly created sub test
         #   case class context.
         # @return [Test::Unit::TestCase] Created sub test case class.
         def sub_test_case(name, &block)
+          parent_test_case = self
           sub_test_case = Class.new(self) do
             singleton_class = class << self; self; end
-            singleton_class.send(:define_method, :name) do
-              name
+            singleton_class.__send__(:define_method, :name) do
+              [parent_test_case.name, name].compact.join("::")
             end
           end
           sub_test_case.class_eval(&block)
           sub_test_case
         end
 
-        # Checkes whether a test that is mathched the query is
+        # Checks whether a test that is matched the query is
         # defined.
         #
         # @option query [String] :path (nil)
@@ -356,32 +371,36 @@ module Test
         # @option query [String] :method_name (nil)
         #   the method name for a test.
         def test_defined?(query)
+          locations = find_locations(query)
+          not locations.empty?
+        end
+
+        def find_locations(query)
           query_path = query[:path]
           query_line = query[:line]
           query_method_name = query[:method_name]
 
-          available_locations = method_locations
-          if query_path
-            available_locations = available_locations.find_all do |location|
-              location[:path].end_with?(query_path)
-            end
-          end
+          available_locations = target_method_locations(query_path)
           if query_line
-            available_location = available_locations.reverse.find do |location|
+            available_locations = available_locations.sort_by do |location|
+              -location[:line]
+            end
+            available_location = available_locations.find do |location|
               query_line >= location[:line]
             end
-            return false if available_location.nil?
+            return [] if available_location.nil?
+            return [] if available_location[:test_case] != self
             available_locations = [available_location]
           end
           if query_method_name
             available_location = available_locations.find do |location|
               query_method_name == location[:method_name]
             end
-            return false if available_location.nil?
+            return [] if available_location.nil?
             available_locations = [available_location]
           end
 
-          not available_locations.empty?
+          available_locations
         end
 
         private
@@ -391,6 +410,28 @@ module Test
         def method_locations
           @@method_locations[self] ||= []
         end
+
+        # @private
+        def target_method_locations(path)
+          if path.nil?
+            self_location = method_locations.first
+            path = self_location[:path] if self_location
+          end
+          return [] if path.nil?
+
+          target_locations = []
+          @@method_locations.each do |test_case, locations|
+            locations.each do |location|
+              absolete_path = File.expand_path(path)
+              location_path = location[:path]
+              location_basename = File.basename(location_path)
+              if location_path == absolete_path or location_basename == path
+                target_locations << location.merge(:test_case => test_case)
+              end
+            end
+          end
+          target_locations
+        end
       end
 
       attr_reader :method_name
@@ -411,9 +452,7 @@ module Test
       def valid? # :nodoc:
         return false unless respond_to?(@method_name)
         test_method = method(@method_name)
-        if @internal_data.have_test_data?
-          return false unless test_method.arity == 1
-        else
+        unless @internal_data.have_test_data?
           return false unless test_method.arity <= 0
         end
         owner = Util::MethodOwnerFinder.find(self, @method_name)
@@ -432,14 +471,31 @@ module Test
           @internal_data.test_started
           yield(STARTED, name)
           yield(STARTED_OBJECT, self)
+          processed_exception_in_setup = false
           begin
-            run_setup
-            run_test
-            run_cleanup
-            add_pass
+            catch do |tag|
+              run_setup do
+                begin
+                  run_test
+                  run_cleanup
+                  add_pass
+                rescue Exception
+                  @internal_data.interrupted
+                  unless handle_exception($!)
+                    processed_exception_in_setup = true
+                    raise
+                  end
+                  throw(tag)
+                end
+              end
+            end
           rescue Exception
-            @internal_data.interrupted
-            raise unless handle_exception($!)
+            if processed_exception_in_setup
+              raise
+            else
+              @internal_data.interrupted
+              raise unless handle_exception($!)
+            end
           ensure
             begin
               run_teardown
@@ -605,10 +661,19 @@ module Test
       # Returns a human-readable name for the specific test that
       # this instance of TestCase represents.
       def name
+        "#{local_name}(#{self.class.name})"
+      end
+
+      # Returns a human-readable name for the specific test that this
+      # instance of TestCase represents.
+      #
+      # `#local_name` doesn't include class name. `#name` includes
+      # class name.
+      def local_name
         if @internal_data.have_test_data?
-          "#{@method_name}[#{data_label}](#{self.class.name})"
+          "#{@method_name}[#{data_label}]"
         else
-          "#{@method_name}(#{self.class.name})"
+          @method_name.to_s
         end
       end
 
@@ -686,13 +751,24 @@ module Test
       end
 
       def run_test
+        signature = "#{self.class}\##{@method_name}"
         redefined_info = self[:redefined]
         if redefined_info
-          notify("#{self.class}\##{@method_name} was redefined",
+          notify("#{signature} was redefined",
                  :backtrace => redefined_info[:backtrace])
         end
         if @internal_data.have_test_data?
-          __send__(@method_name, @internal_data.test_data)
+          test_method = method(@method_name)
+          if test_method.arity == 1 or test_method.arity < 0
+            __send__(@method_name, @internal_data.test_data)
+          else
+            locations = self.class.find_locations(:method_name => @method_name)
+            backtrace = locations.collect do |location|
+              "#{location[:path]}:#{location[:line]}"
+            end
+            notify("#{signature} misses a parameter to take test data",
+                   :backtrace => backtrace)
+          end
         else
           __send__(@method_name)
         end
@@ -703,7 +779,7 @@ module Test
           if handler.respond_to?(:call)
             handled = handler.call(self, exception)
           else
-            handled = send(handler, exception)
+            handled = __send__(handler, exception)
           end
           return true if handled
         end
