@@ -1,5 +1,5 @@
 # Copyright (C) 2009  Yuto Hayamizu <y.hayamizu@gmail.com>
-# Copyright (C) 2010-2011  Kouhei Sutou <kou@clear-code.com>
+# Copyright (C) 2010-2019  Kouhei Sutou <kou@clear-code.com>
 #
 # This library is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -86,7 +86,7 @@ class TestClient < Test::Unit::TestCase
   def test_listen
     port = 12345
     @client.connection_spec = "inet:#{port}"
-    assert_raise(Errno::ECONNREFUSED) do
+    assert_raise_kind_of(SystemCallError) do
       TCPSocket.new("localhost", port)
     end
     @client.listen
