@@ -740,8 +740,11 @@ feed (const gchar *packet, gsize packet_size)
     GError *error = NULL;
 
     milter_client_context_feed(context, packet, packet_size, &error);
-    if (!error)
+    if (!error) {
+        if (MILTER_IS_LIBEV_EVENT_LOOP(loop))
+            cut_omit("MilterLibevEventLoop doesn't support GCutStringIOChannel.");
         milter_test_pump_all_events(loop);
+    }
 
     return error;
 }
@@ -1683,6 +1686,9 @@ test_write_error (void)
 {
     GError *error = NULL;
 
+    if (MILTER_IS_LIBEV_EVENT_LOOP(loop))
+        cut_omit("MilterLibevEventLoop doesn't support GCutStringIOChannel.");
+
     g_io_channel_set_buffered(channel, FALSE);
     g_io_channel_set_flags(channel, G_IO_FLAG_NONBLOCK, &error);
     gcut_assert_error(error);
